# electric-nlp.mod : nlp formulation
###############################################################################
# Sets used to define the problem                                             #
###############################################################################

set F;					# Firms
param Fs{F} binary;			# Stackelberg leaders
param Fc{F} binary;			# Cournot follower firms
param Fp{F} binary;			# Price-taker follower firms
param Ff{f in F} := min(Fc[f]+Fp[f],1);	# All followers

check {f in F}: Fs[f]+Fc[f]+Fp[f] == 1;	# Firms assigned to disjoint sets
check: sum{f in F} Fs[f] <= 1;		# At most one leader

set I;				# Nodes in the network
set ARCS within I cross I;	# Arcs (interfaces) in the transmission network

set P;				# Plants
set H{I,F} within P;		# Set of Plants for firm F at node I

set T;				# Time periods

set K;				# Loops for Kirchhoff's laws
set LOOP{K} within ARCS;	# Arcs associated with the loop

###############################################################################
# Parameters                                                                  #
###############################################################################

param OTC{P} binary;		# Flag for plants subject to NOx cap
param C{P};			# Marginal cost 
param E{P};			# Emission rate
param CAP{P};			# Production capacity

param N{F};			# NOx allowances for the firm

param Tlim{ARCS};		# Thermal limit of transmission
param R{ARCS};			# Reactance
param L{ARCS};			# Loss coefficient

param Bw{T};			# Block width of load duration

param P0{I, T};			# Vertical intercept of demand curve
param P1{I, T};			# Vertical intercept of demand curve
param Q0{I, T};			# Horizontal intercept of demand curve
param Q1{I, T};			# Horizontal intercept of demand curve
param  Z{I, T};

param Withhold;

param comp_bound;

###############################################################################
# Variables in the problem                                                    #
###############################################################################

var s{F, I, T} >= 0;				# Power sold
var x{f in F, i in I, h in H[i,f], T} >= 0, <= CAP[h];	# Output of plant
var flow{(i,j) in ARCS, T} >= 0, <= Tlim[i,j];		# Flow in the arc
var y{I, T};				# Power delivered
var w{I, T};				# Wheeling charge
var a{I, T};				# Power purchased/sold by arbitrager
var q{I, T} >= 0;				# Power purchased/sold by ISO

var pN >= 0;					# NOx permit charge
var nw{F} >= 0;				# Permit withholding

###############################################################################
# Mutipliers for variational inequality constraints                           #
###############################################################################

var theta{F, T};
var gamma{I, T} >= 0;
var tau{K, T};
var delta{T};
var mu{T};

###############################################################################
# Defined variables                                                           #
###############################################################################

var pE{i in I, t in T} = 
   P0[i,t] - P0[i,t]/Q0[i,t]*(sum{f in F} s[f,i,t] - q[i,t] + a[i,t]);
  
###############################################################################
# Additional slacks                                                           #
###############################################################################

var sc{f in F, i in I, t in T : 1 == Ff[f]} >= 0;
var sw{f in F, i in I, h in H[i,f], t in T : 1 == Ff[f] and CAP[h] > 0} >= 0;
var sv{f in F, i in I, h in H[i,f], t in T : 1 == Ff[f] and CAP[h] > 0} >= 0;
var sq{i in I, t in T} >= 0;
var sfw{(i,j) in ARCS, t in T: Tlim[i,j] > 0} >= 0;
var sfv{(i,j) in ARCS, t in T: Tlim[i,j] > 0} >= 0;
var sg{i in I, t in T} >= 0;
var sp >= 0;

###############################################################################
# Model                                                                       #
###############################################################################

minimize cost: sum{f in F : 1 == Fs[f]} (
  -sum{i in I, t in T} (pE[i,t]-w[i,t])*s[f,i,t]*Bw[t]
  +sum{i in I, h in H[i,f], t in T} (C[h]-w[i,t])*x[f,i,h,t]*Bw[t]
  +pN*(sum{i in I, h in H[i,f], t in T : 1 == OTC[h]} E[h]*x[f,i,h,t]*Bw[t] - N[f] + nw[f])
);

subject to s_conserve{f in F, t in T : 1 == Fs[f]}:
  sum{i in I, h in H[i,f]} x[f,i,h,t] - sum{i in I} s[f,i,t] = 0;

subject to cournot{f in F, i in I, t in T : 1 == Ff[f]}:
  sc[f,i,t] = -pE[i,t] + w[i,t] + (if (1 == Fc[f]) then P0[i,t]/Q0[i,t]*s[f,i,t] else 0) + theta[f,t];

subject to supply{f in F, i in I, h in H[i,f], t in T : 1 == Ff[f] and CAP[h] > 0}:
  sw[f,i,h,t] - sv[f,i,h,t] = C[h] - w[i,t] + (if (1 == OTC[h]) then pN*E[h] else 0) - theta[f,t];

subject to conserve{f in F, t in T : 1 == Ff[f]}:
  sum{i in I, h in H[i,f]} x[f,i,h,t] - sum{i in I} s[f,i,t] = 0;

subject to iso_optim_y {i in I, t in T}:
  -w[i,t]*Bw[t] + gamma[i,t] + delta[t] = 0;

subject to iso_optim_q {i in I, t in T}:
  sq[i,t] = pE[i,t]*Bw[t] - gamma[i,t];

subject to iso_optim_flow{(i,j) in ARCS, t in T: Tlim[i,j] > 0}:
  sfw[i,j,t] - sfv[i,j,t] = gamma[i,t] - gamma[j,t]*(1-2*L[i,j]*flow[i,j,t]) + 
                               sum {k in K: (i,j) in LOOP[k]} tau[k,t]*R[i,j] - 
                               sum {k in K: (j,i) in LOOP[k]} tau[k,t]*R[j,i];

subject to iso_current{i in I, t in T}:
  sg[i,t] = - y[i,t] + q[i,t] - sum{j in I: (i,j) in ARCS} (flow[i,j,t]-(1 - L[j,i]*flow[j,i,t])*flow[j,i,t]);

subject to iso_voltage{k in K, t in T}:
  0 = sum {(i,j) in LOOP[k]} (flow[i,j,t] - flow[j,i,t])*R[i,j];

#subject to iso_y {t in T}:
#  0 = sum{i in I} y[i,t];

subject to iso_y {t in T}:
  mu[t] = pE[14,t];

# Can make the following substitution
#subject to iso_y {t in T}:
#  pE[14,t] = mu[t];

subject to arbitrage_optim{i in I, t in T}:
  -pE[i,t] + w[i,t] + mu[t] = 0;

subject to arbitrage_con{t in T}:
  0 = sum{i in I} a[i,t];

subject to service{i in I, t in T}:
  y[i,t] + sum{f in F, h in H[i,f]} x[f,i,h,t] = sum{f in F} s[f,i,t] + a[i,t];

subject to market:
  sp = -sum{f in F} (sum{i in I, h in H[i,f], t in T : 1 == OTC[h]} E[h]*Bw[t]*x[f,i,h,t] - N[f]) - sum{f in F} nw[f];

### complementarity constraint
  compl_c: sum{f in F, i in I, t in T : 1 == Ff[f]}              s[f,i,t]*sc[f,i,t] <= comp_bound;
  compl_s: sum{f in F, i in I, h in H[i,f], t in T : 1 == Ff[f] and CAP[h] > 0} (x[f,i,h,t]*sw[f,i,h,t] + (CAP[h] - x[f,i,h,t])*sv[f,i,h,t]) <= comp_bound;
  compl_q: sum{i in I, t in T}                                   q[i,t]*sq[i,t] <= comp_bound;
  compl_f: sum{(i,j) in ARCS, t in T: Tlim[i,j] > 0} (flow[i,j,t]*sfw[i,j,t] + 
           (Tlim[i,j] - flow[i,j,t])*sfv[i,j,t]) <= comp_bound;
  compl_g: sum{i in I, t in T}                                   gamma[i,t]*sg[i,t] <= comp_bound;
  compl_p:	                                                 sp*pN <= comp_bound;
